const express = require('express');
const mysql = require('mysql');
const blood_request_responses = express();
// Middleware for parsing JSON
blood_request_responses.use(express.json());
//DB configure-connection
const connection = require('../config');

// GET all method 
blood_request_responses.get('/request', (req, res) => {
  const query = 'SELECT * FROM blood_request_responses';

  connection.query(query, (err, results) => {
    if (err) {
      console.error('Error executing MySQL query: ', err);
      res.sendStatus(500);
      return;
    }

    res.json(results);
  });
});
//filter-get by applied_by 
blood_request_responses.get('/request-appliedby/:appliedById', (req, res) => {
  // Get the 'applied_by' ID from the URL parameters
  const appliedById = req.params.appliedById;

  // Construct the SQL query with a WHERE clause to filter by 'applied_by' ID
  const query = 'SELECT * FROM blood_request_responses WHERE applied_by = ?';

  // Execute the query with the provided 'applied_by' ID
  connection.query(query, [appliedById], (err, results) => {
    if (err) {
      console.error('Error executing MySQL query: ', err);
      res.sendStatus(500);
      return;
    }

    res.json(results);
  });
});
//claim_detail api list

blood_request_responses.get('/requestid/:appliedById', (req, res) => {
  // Get the 'applied_by' ID from the URL parameters
  const appliedById = req.params.appliedById;

  // Construct the SQL query to retrieve response data
  const responseQuery = `
    SELECT br.*, u.id AS user_id, u.name AS name, u.mobile_number,u.email
    FROM blood_request_responses AS br
    JOIN users AS u ON br.applied_by = u.id
    WHERE br.applied_by = ? AND br.claim_status = 0`;

  // Execute the query to get response data
  connection.query(responseQuery, [appliedById], (err, responseResults) => {
    if (err) {
      console.error('Error executing MySQL query for responses: ', err);
      res.sendStatus(500);
      return;
    }

    // Check if there are no results with claim_status equal to 0
    if (responseResults.length === 0) {
      // If there are no results, return an empty array
      res.json([]);
      return;
    }

    // Extract an array of unique blood_request_id values from the response results
    const bloodRequestIds = [...new Set(responseResults.map(result => result.blood_request_id))];

    // Construct a SQL query to retrieve blood_request data based on the blood_request_id values
    const bloodRequestQuery = `
      SELECT id AS blood_request_id, patient_name, patient_age, hospital_name, hospital_address, hospital_city, hospital_state
      FROM blood_request
      WHERE id IN (?)`;

    // Execute the query to get blood_request data
    connection.query(bloodRequestQuery, [bloodRequestIds], (err, bloodRequestResults) => {
      if (err) {
        console.error('Error executing MySQL query for blood requests: ', err);
        res.sendStatus(500);
        return;
      }

      // Create a map to quickly access blood_request data by blood_request_id
      const bloodRequestMap = new Map();
      for (const bloodRequest of bloodRequestResults) {
        bloodRequestMap.set(bloodRequest.blood_request_id, bloodRequest);
      }

      // Merge the response results with the corresponding blood_request data
      const mergedResults = responseResults.map(result => ({
        ...result,
        blood_request: bloodRequestMap.get(result.blood_request_id),
      }));

      res.json(mergedResults);
    });
  });
});



// GET bu Id method
blood_request_responses.get('/request/:id', (req, res) => {
  const requestId = req.params.id;
  const query = 'SELECT * FROM blood_request_responses WHERE id = ?';

  connection.query(query, [requestId], (err, results) => {
    if (err) {
      console.error('Error executing MySQL query: ', err);
      res.sendStatus(500);
      return;
    }

    if (results.length === 0) {
      res.sendStatus(404);
      return;
    }

    res.json(results[0]);
  });
});

blood_request_responses.post('/add-request', (req, res) => {
  const {
    blood_request_id,
    applied_by,
    applied_date,
    status_request,
    status_donar
  } = req.body;

  const query = `INSERT INTO blood_request_responses (
    blood_request_id,
    applied_by,
    applied_date,
    status_request,
    status_donar,
    created_at,
    updated_at
  ) VALUES (?, ?, ?, ?, ?, NOW(), NOW())`;

  const statusRequestValue = status_request === "null" ? null : status_request;

  connection.query(
    query,
    [
      blood_request_id,
      applied_by,
      applied_date,
      statusRequestValue,
      status_donar
    ],
    (err, results) => {
      if (err) {
        console.error('Error executing MySQL query: ', err);
        res.sendStatus(500);
        return;
      }

      res.sendStatus(200);
    }
  );
});


// PUT method
blood_request_responses.put('/edit-request/:id', (req, res) => {
  const requestId = req.params.id;
  const {
    applied_by,
    applied_date,
    status_request,
    status_donar
  } = req.body;

  const formattedDate = new Date(applied_date).toISOString().split('T')[0]; // Extract the date part

  const query = `UPDATE blood_request_responses SET
    applied_by = ?,
    applied_date = ?,
    status_request = ?,
    status_donar = ?,
    updated_at = NOW()
    WHERE id = ?`;

  connection.query(
    query,
    [
      applied_by,
      formattedDate,
      status_request,
      status_donar,
      requestId
    ],
    (err, results) => {
      if (err) {
        console.error('Error executing MySQL query:', err);
        res.sendStatus(500);
        return;
      }

      if (results.affectedRows === 0) {
        res.sendStatus(404);
        return;
      }

      res.sendStatus(200);
    }
  );
});


// DELETE method
blood_request_responses.delete('/delete-request/:id', (req, res) => {
  const requestId = req.params.id;
  const query = 'DELETE FROM blood_request_responses WHERE id = ?';

  connection.query(query, [requestId], (err, results) => {
    if (err) {
      console.error('Error executing MySQL query: ', err);
      res.sendStatus(500);
      return;
    }

    if (results.affectedRows === 0) {
      res.sendStatus(404);
      return;
    }

    res.sendStatus(200);
  });
});

// GET method to list all matching user_id in users_donar_profile
blood_request_responses.get('/bloodresponserelate/:user_id', (req, res) => {
  const appliedby = req.params.user_id;

  // Query to fetch the user's information and matching profiles
  const query = 'SELECT u.*, brr.* FROM users u LEFT JOIN blood_request_responses brr ON u.id = brr.applied_by WHERE u.id = ?';

  connection.query(query, [appliedby], (err, results) => {
    if (err) {
      console.error('Error executing MySQL query: ', err);
      res.sendStatus(500);
      return;
    }

    if (results.length === 0) {
      res.sendStatus(404);
      return;
    }

    const bloodrequestresponses = {
      id: results[0].id,
      name: results[0].name,
      mobile_number: results[0].mobile_number,
      date_of_birth: results[0].date_of_birth,
      gender: results[0].gender,
      email: results[0].email,
      matching_profiles: []
    };

    results.forEach(result => {
      if (result.applied_by) {
        const detail = {
          id: result.id,
          applied_by:result.applied_by,
          applied_date:result.applied_date,
          status_request:result.status_request,
          status_donar:result.status_donar,
          created_at: result.created_at,
          updated_at: result.updated_at
        };

        bloodrequestresponses.matching_profiles.push(detail);
      }
    });

    res.json(bloodrequestresponses);
  });
});

module.exports = blood_request_responses;